from sceneNode import SceneNode
from rect import Rect
import vector

class Background(SceneNode):
    def __init__(self, parent, type = None, name = None):
        SceneNode.__init__(self, parent, type, name)
        
        self.addInitDepend("gfxServer")
        
        self.defaultVar("parallaxFactor",-0.5)
        self.defaultVar("scroll",0.0)
        
    def init(self):
        self.gfxServer = self.getFirstObjectOfName("gfxServer")
        if self.gfxServer:
            self.gfxServer.earlyObjects.append(self)
            self.screenw = self.gfxServer.getVar("screenw")
            self.screenh = self.gfxServer.getVar("screenh")
        else:
            self.screenw = 800
            self.screenh = 600
            
        self.player = self.getFirstObjectOfName("player")
        self.lastPlayerPos = self.player.getVar("pos")
            
        for dx in range(-1,1):
            for dy in range(-1,1):
                rect = Rect(self)
                rect.setVar("pos",(self.screenw * dx, self.screenh * dy,0))
                rect.setVar("scale",(self.screenw,self.screenh,1))
                rect.setVar("texture","background.png")
                
        self.scrollVec = vector.dot(self.getVar("scroll"),(self.screenw,self.screenh,1))

    def update(self, time, dtime):
        vel = vector.scale(self.scrollVec,dtime)
        pos = self.getVar("pos")
        pos = vector.add(pos,vel)
        pos = vector.fix(pos,(0,0,0),(self.screenw,self.screenh,0))
        self.setVar("pos",pos)
