from sceneNode import SceneNode

import math, vector

class Bullet(SceneNode):
    def __init__(self, parent, type = None, name = None):
        SceneNode.__init__(self,parent,type,name)

        self.defaultVar("lifetime",5.0)
        self.defaultVar("vel",(0,-500))
        self.defaultVar("damage",10)
        self.defaultVar("hitRadius",5)
        self.defaultVar("friendly",False)
        self.defaultVar("firer",None)
        self.defaultVar("target",None)
        self.defaultVar("seekTarget",0)
        
        self.startTime = None
        
        shmup = self.getFirstObjectOfName("shmup")
        self.enemies = shmup.getChild("enemies")
        self.player = shmup.getChild("player")
        
        self.gfx = self.makeGfx()
        self.addInitDepend("gfxServer")
        
        self.addInitDepend("soundServer")
        
    def init(self):
        self.gfxServer = self.getFirstObjectOfName("gfxServer")
        self.explosionServer = self.getFirstObjectOfName("explosionServer")
        self.screenw = self.gfxServer.getVar("screenw")
        self.screenh = self.gfxServer.getVar("screenh")
        
        self.soundServer = self.getFirstObjectOfName("soundServer")
        #self.soundServer.messagePlay(("laser.wav",0.2))
        
    def makeGfx(self):
        return None
        
    def update(self, time, dtime):
        if not self.startTime:
            self.startTime = time
            
        x,y,z = self.getVar("pos")
            
        if time - self.startTime > self.getVar("lifetime") or x < 0 or y < 0 or x > self.screenw or y > self.screenh:
            if self.getVar("friendly"):
                for enemy in self.enemies.children.itervalues():
                    if not enemy.getVar("done"):
                        enemy.receiveMessage("bulletMissed")
            
            self.deleteThis()
            
        if self.gfx:
            color = (time - self.startTime) / self.getVar("lifetime")
            for child in self.gfx.children.itervalues():
                self.gfx.setVar("centerColor",(color,color,color))

        velx, vely = self.getVar("vel")
        
        target = self.getObject(self.getVar("target"))
        seekTarget = self.getVar("seekTarget")
        #self.log("target %s seekTarget %f" % (self.getVar("target"),seekTarget))
        if target and seekTarget:
            currentTh = vector.th((velx,vely))
            speed = vector.length((velx,vely))
            destTh = vector.thTo(self.getVar("pos"),target.getVar("pos"))
            diffTh = destTh - currentTh
            while diffTh < -math.pi:
                diffTh += math.pi * 2
            while diffTh > math.pi:
                diffTh -= math.pi * 2
            newTh = currentTh + diffTh * dtime * seekTarget #* 0.5
            velx = speed * math.cos(newTh)
            vely = speed * math.sin(newTh)
        
        posx, posy, posz = self.getVar("pos")
        posx += velx * dtime
        posy += vely * dtime
        self.setVar("vel",(velx,vely))
        self.setVar("pos",(posx,posy,posz))
        
        if self.getVar("friendly"):
            for enemy in self.enemies.children.itervalues():
                if self.dist(self.getVar("pos"),enemy.getVar("pos")) < self.getVar("hitRadius") + enemy.getVar("hitRadius"):
                    self.explode()
                    enemy.receiveMessage("damage",(self.getVar("damage"),self.getVar("firer")))
                    if self.getVar("firer"):
                        firer = self.getObject(self.getVar("firer"))
                        firer.receiveMessage("didDamage",(self.getVar("damage"),enemy.path))
                        
        else:
            if self.dist(self.getVar("pos"),self.player.getVar("pos")) < self.getVar("hitRadius") + self.player.getVar("hitRadius"):
                self.explode()
                self.player.receiveMessage("damage",(self.getVar("damage"),self.getVar("firer")))
                if self.getVar("firer"):
                    firer = self.getObject(self.getVar("firer"))
                    firer.receiveMessage("didDamage",(self.getVar("damage"),self.player.path))
                    
    def explode(self):
        if not self.getVar("friendly"):
            self.explosionServer.receiveMessage("spawn",(self.getVar("pos"),10,0.5))
        self.deleteThis()
        
    def dist(self, v1, v2):
        dx = v2[0] - v1[0]
        dy = v2[1] - v1[1]
        return math.sqrt(dx * dx + dy * dy)
