from object import Object
from bullet import Bullet
from dynLoad import dynLoad

import math, vector

class BulletEmitter(Object):
    def __init__(self, parent, type = None, name = None):
        Object.__init__(self, parent, type, name)
        
        self.defaultVar("friendly",False)
        self.setVar("firing",False)
        
        self.defaultVar("fireType","straight")
        self.defaultVar("fireRate",1.0)
        self.defaultVar("bulletType","standardBullet")
        self.defaultVar("bulletSpeed",100)
        self.defaultVar("numSpread",3)
        self.defaultVar("sound","")
        self.defaultVar("soundVolume",0.5)
        
        self.fireStartTime = None
        self.sweepCount = 0
        
        self.addInitDepend("shmup")
        self.addInitDepend("gfxServer")
        self.addInitDepend("soundServer")
        
    def init(self):
        
        self.gfxServer = self.getFirstObjectOfName("gfxServer")
        self.screenw = self.gfxServer.getVar("screenw")
        self.screenh = self.gfxServer.getVar("screenh")
        
        self.bullets = self.getFirstObjectOfName("shmup").getChild("bullets")
        self.enemies = self.getFirstObjectOfName("shmup").getChild("enemies")
        self.player = self.getFirstObjectOfName("player")
        
        self.soundServer = self.getFirstObjectOfName("soundServer")
        
        self.bulletClass = dynLoad(self.getVar("bulletType")) or Bullet
        
    def playSound(self):
        sound = self.getVar("sound")
        if sound and self.soundServer:
            self.soundServer.receiveMessage("play",(sound,self.getVar("soundVolume")))
            
    def update(self, time, dtime):
        x, y, z = self.parent.getVar("pos")
        if self.getVar("firing") and x > 0 and x < self.screenw and y > 0 and y < self.screenh:
            
            fireType = self.getVar("fireType")
            bulletType = self.getVar("bulletType")
            friendly = self.getVar("friendly")
            bulletSpeed = self.getVar("bulletSpeed")
            numSpread = self.getVar("numSpread")
            
            if friendly:
                minDist = None
                target = None
                targetPath = None
                for enemy in self.enemies.children.itervalues():
                    ex, ey, ez = epos = enemy.getVar("pos")
                    if not enemy.getVar("done") and ex > 0 and ex < self.screenw and ey > 0 and ey < self.screenh:
                        dist = vector.dist(self.parent.getVar("pos"),epos)
                        if not minDist or dist < minDist:
                            minDist = dist
                            target = enemy
                            targetPath = enemy.path
                            
                #if not target: return
                
            else:
                target = self.player
                targetPath = self.player.path
                        
            #self.log("firing with pattern %s" % fireType)
            
            if fireType == 'straight':
                
                if not self.fireStartTime or time - self.fireStartTime > self.getVar("fireRate"):
                    self.fireStartTime = time
                    bullet = self.bulletClass(self.bullets,bulletType)
                    bullet.setVar("pos",self.parent.getVar("pos"))
                    bullet.setVar("firer",self.parent.path)
                    bullet.setVar("target",targetPath)
                    bullet.setVar("friendly",friendly)
                    if friendly:
                        bullet.setVar("vel",(0,-bulletSpeed))
                    else:
                        bullet.setVar("vel",(0,bulletSpeed))
                    if self.hasVar("seekTarget"):
                        bullet.setVar("seekTarget",self.getVar("seekTarget"))
                        
                    self.playSound()
                        
            elif fireType == 'spread':
                if not self.fireStartTime or time - self.fireStartTime > self.getVar("fireRate") * numSpread:
                    self.fireStartTime = time
                    for i in range(1,numSpread+1):
                        th = float(i) / float(numSpread+1) * math.pi
                        dx = bulletSpeed * math.cos(th)
                        dy = bulletSpeed * math.sin(th)
                        if friendly:
                            dy *= -1
                            
                        bullet = self.bulletClass(self.bullets,bulletType)
                        bullet.setVar("pos",self.parent.getVar("pos"))
                        bullet.setVar("firer",self.parent.path)
                        bullet.setVar("friendly",friendly)
                        bullet.setVar("target",targetPath)
                        bullet.setVar("vel",(dx,dy))
                        if self.hasVar("seekTarget"):
                            bullet.setVar("seekTarget",self.getVar("seekTarget"))
                        
                    self.playSound()
                        
            elif fireType == 'targeted':
                if not self.fireStartTime or time - self.fireStartTime > self.getVar("fireRate"):
                    self.fireStartTime = time
                    
                    if target:
                        th = vector.thTo(self.parent.getVar("pos"),target.getVar("pos"))
                    else:
                        th = -math.pi/2
                    dx = bulletSpeed * math.cos(th)
                    dy = bulletSpeed * math.sin(th)
                    bullet = self.bulletClass(self.bullets,bulletType)
                    bullet.setVar("pos",self.parent.getVar("pos"))
                    bullet.setVar("firer",self.parent.path)
                    bullet.setVar("target",targetPath)
                    bullet.setVar("friendly",friendly)
                    bullet.setVar("vel",(dx,dy))
                    if self.hasVar("seekTarget"):
                        bullet.setVar("seekTarget",self.getVar("seekTarget"))
                        
                    self.playSound()
                    
            elif fireType == 'sweep':
                if not self.fireStartTime or time - self.fireStartTime > self.getVar("fireRate"):
                    self.fireStartTime = time
                    
                    th = float(self.sweepCount+1) / float(numSpread+1) * math.pi
                    self.sweepCount += 1
                    if self.sweepCount >= numSpread:
                        self.sweepCount = 0
                        
                    dx = bulletSpeed * math.cos(th)
                    dy = bulletSpeed * math.sin(th)
                    if friendly:
                        dy *= -1
                        
                    bullet = self.bulletClass(self.bullets,bulletType)
                    bullet.setVar("pos",self.parent.getVar("pos"))
                    bullet.setVar("firer",self.parent.path)
                    bullet.setVar("target",targetPath)
                    bullet.setVar("friendly",friendly)
                    bullet.setVar("vel",(dx,dy))
                    if self.hasVar("seekTarget"):
                        bullet.setVar("seekTarget",self.getVar("seekTarget"))
                        
                    self.playSound()
