from object import Object
import curses

class CursesGfxServer(Object):
    def __init__(self, parent):
        Object.__init__(self, parent, "cursesGfxServer", "gfxServer")
        
    def init(self):
        self.screen = curses.initscr()
        curses.cbreak()
        curses.noecho()
        self.screen.keypad(1)
        
        self.screenh, self.screenw = self.screen.getmaxyx()
        self.setVar('screenh',self.screenh)
        self.setVar('screenw',self.screenw)
        
        self.setVar('camerax',0)
        self.setVar('cameray',0)
        
    def destroy(self):
        curses.nocbreak()
        self.screen.keypad(0)
        curses.echo()
        curses.endwin()
        
    def preRender(self):
        self.screen.clear()
        
    def render(self, obj):
        if obj.active and obj.initialized and not obj.destroyed:
            camerax = self.getVar('camerax')
            cameray = self.getVar('cameray')
            if obj.hasVar("pos"):
                x, y, z = obj.getVar("pos")
                x -= camerax
                y -= cameray
                if x >= 0 and x < self.screenw and y >= 0 and y < self.screenh:
                    if obj.hasVar('ch'):
                        self.screen.addch(y,x,obj.getVar('ch'))
                    elif obj.hasVar('str') and x < self.screenw - len(obj.getVar('str')):
                        self.screen.addstr(y,x,obj.getVar('str'))
                        
            for child in obj.children.itervalues():
                self.render(child)

    def postRender(self):
        self.screen.move(0,0)
        self.screen.refresh()

