from sceneNode import SceneNode
from circle import Circle
from rect import Rect
from bulletEmitter import BulletEmitter

import math
import random

class Enemy(SceneNode):
    def __init__(self, parent, type = None, name = None):
        SceneNode.__init__(self, parent, type, name)
        
        self.defaultVar("startPos",(0,0))
        self.defaultVar("stopPos",(1,0))
        self.defaultVar("control1",(0.75,0.25))
        self.defaultVar("control2",(0.25,0.25))
        self.defaultVar("lifetime",10.0)
        
        self.defaultVar("hitRadius",20)
        self.defaultVar("maxHp",100)
        
        self.setVar("hp",self.getVar("maxHp"))
        self.setVar("done",True)
        if random.randint(0,1):
            self.setVar("direction","left")
        else:
            self.setVar("direction","right")
        
        self.startTime = None
        self.screenw = 800
        self.screenh = 600
        self.active = False
        
        #self.gfx = Circle(self)
        #hitRadius = self.getVar("hitRadius")
        #self.gfx.setVar("scale",(hitRadius,hitRadius,1))
        #self.gfx.setVar("color",(1,0,0))
        
        self.gfx = Rect(self)
        hitRadius = self.getVar("hitRadius")
        self.gfx.setVar("pos",(-hitRadius,-hitRadius,0))
        self.gfx.setVar("scale",(hitRadius*2,hitRadius*2,1))
        self.gfx.setVar("texture","enemy.png")
        
        self.bulletEmitter = BulletEmitter(self,"enemyBulletEmitter")
        self.bulletEmitter.setVar("firing",True)
        
        self.addInitDepend("gfxServer")
        self.addInitDepend("player")
        
        self.fitness = 0

    def init(self):
        self.gfxServer = self.getFirstObjectOfName("gfxServer")
        if self.gfxServer:
            self.screenw = self.gfxServer.getVar("screenw")
            self.screenh = self.gfxServer.getVar("screenh")
            
        self.player = self.getFirstObjectOfName("player")
        self.explosionServer = self.getFirstObjectOfName("explosionServer")
            
    def update(self, time, dtime):
        lifetime = self.getVar("lifetime")
        if not self.startTime:
            self.startTime = time
            
        if self.getVar("done") or time - self.startTime > lifetime:
            self.finish()
            return
            
        a = (time - self.startTime) / lifetime
        b = 1.0 - a
        
        ax, ay = self.getVar("startPos")
        bx, by = self.getVar("control1")
        cx, cy = self.getVar("control2")
        dx, dy = self.getVar("stopPos")
        
        ax *= float(self.screenw)
        ay *= float(self.screenh)
        bx *= float(self.screenw)
        by *= float(self.screenh)
        cx *= float(self.screenw)
        cy *= float(self.screenh)
        dx *= float(self.screenw)
        dy *= float(self.screenh)
        
        x = ax * a * a * a + bx * 3 * a * a * b + cx * 3 * a * b * b + dx * b * b * b
        y = ay * a * a * a + by * 3 * a * a * b + cy * 3 * a * b * b + dy * b * b * b
        
        if self.getVar("direction") == "right":
            x = x * -1 + self.screenw
        
        self.setVar("pos",(x,y,0))
        
        fitness = 0
        
        #if x > 0 and x < self.screenw and y > 0 and y < self.screenh:
            #distToPlayer = self.dist(self.getVar("pos"),self.player.getVar("pos"))
            #self.fitness += distToPlayer
        
        self.fitness += fitness * dtime
        
    def messageDamage(self, msg):
        damage, firer = msg
        self.fitness -= damage
        self.setVar("hp",self.getVar("hp") - damage)
        if self.getVar("hp") <= 0:
            self.player.receiveMessage("killed",self.path)
            self.explosionServer.receiveMessage("spawn",(self.getVar("pos"),25,0.5))
            self.fitness -= 1000
            self.setVar("pos",(-100,-100,0))
            self.setVar("done",True)
        
    def messageDidDamage(self, msg):
        damage, firer = msg
        self.fitness += damage * 10
        
    def messageBulletMissed(self, msg):
        self.fitness += 1
        
    def dist(self, v1, v2):
        dx = v2[0] - v1[0]
        dy = v2[1] - v1[1]
        return math.sqrt(dx * dx + dy * dy)

    def reset(self):
        self.setVar("done",False)
        self.active = True
        self.startTime = 0
        self.fitness = 0
        self.setVar("hp",self.getVar("maxHp"))
        
    def finish(self):
        self.setVar("done",True)
        self.setVar("pos",(-100,-100,0))
        self.active = False
        
    def getFitness(self):
        return self.fitness
