from sceneNode import SceneNode
from rect import Rect

class Explosion(SceneNode):
    def __init__(self, parent, type = None, name = None):
        SceneNode.__init__(self, parent, type, name)
        
        self.defaultVar("lifetime",0.2)
        self.defaultVar("filenameTemplate","explosion%d.png")
        self.defaultVar("numFrames",3)
        self.defaultVar("size",10.0)
        
        self.addInitDepend("soundServer")
        
    def init(self):
        self.soundServer = self.getFirstObjectOfName("soundServer")
        if self.soundServer:
            self.soundServer.receiveMessage("play",("explosion.wav",1.0))
        
        self.frameNum = 0
        self.frameLifetime = self.getVar("lifetime") / float(self.getVar("numFrames"))
        self.frameStartTime = None
        
        size = self.getVar("size")
        self.rect = Rect(self)
        self.rect.setVar("pos",(-size,-size,0))
        self.rect.setVar("scale",(size*2,size*2,1))
        self.rect.setVar("texture",self.getVar("filenameTemplate") % self.frameNum)
        
    def update(self, time, dtime):
        if not self.frameStartTime:
            self.frameStartTime = time
            
        if time - self.frameStartTime > self.frameLifetime:
            self.frameStartTime = time
            
            self.frameNum += 1
            if self.frameNum >= self.getVar("numFrames"):
                self.deleteThis()
                return
            else:
                self.rect.setVar("texture",self.getVar("filenameTemplate") % self.frameNum)
