from object import Object
import random, copy, binary

class Ga(Object):
    def __init__(self, parent, type = None, name = None):
        Object.__init__(self,parent, type, name)
        
        self.defaultVar("popSize",20)
        self.defaultVar("genomeSize",0)
        self.defaultVar("numGens",-1)
        self.defaultVar("selection","tournament")
        self.defaultVar("replacement","simple")
        self.defaultVar("crossover","simple")
        self.defaultVar("mutation","simple")
        self.defaultVar("replacement","simple")
        self.defaultVar("crossoverProb",0.9)
        self.defaultVar("mutationProb",0.01)
        self.defaultVar("fitnessSharing",False)
        self.defaultVar("seed",0)
        
        self.setVar("running",False)
        
        self.startGen = False
        
        self.varOffsets = {}
        self.varTypes = {}
        self.varData = {}
        
    def registerFloat(self, name, min, max, resolution):
        range = int((max - min) / resolution) + 1
        size = binary.minBits(range)
        self.registerFloatBits(name,min,max,size)
        
    def registerFloatBits(self, name, min, max, size):
        self.varOffsets[name] = self.getVar("genomeSize")
        self.varTypes[name] = 'float'
        self.varData[name] = (min,max,size)
        self.setVar("genomeSize",self.getVar("genomeSize")+size)
        
    def getFloat(self, ind, name):
        if self.varTypes.has_key(name) and self.varTypes[name] == 'float':
            pos = self.varOffsets[name]
            min, max, size = self.varData[name]
            return binary.binToFloat(ind,min,max,pos,size)
        else:
            return None
        
    def registerInt(self, name, min, max):
        size = binary.minBits(max - min)
        
        self.varOffsets[name] = self.getVar("genomeSize")
        self.varTypes[name] = 'int'
        self.varData[name] = (min,max,size)
        self.setVar("genomeSize",self.getVar("genomeSize")+size)
        
    def getInt(self, ind, name):
        if self.varTypes.has_key(name) and self.varTypes[name] == 'int':
            pos = self.varOffsets[name]
            min, max, size = self.varData[name]
            return int(binary.binToFloat(ind,min,max,pos,size))
        else:
            return None
        
    def registerEnum(self, name, enum):
        size = binary.minBits(len(enum))
        
        self.varOffsets[name] = self.getVar("genomeSize")
        self.varTypes[name] = 'enum'
        self.varData[name] = (enum, size)
        self.setVar("genomeSize",self.getVar("genomeSize")+size)
        
    def getEnum(self, ind, name):
        if self.varTypes.has_key(name) and self.varTypes[name] == 'enum':
            pos = self.varOffsets[name]
            enum, size = self.varData[name]
            return binary.binToEnum(ind,enum,pos,size)
        else:
            return None
        
    def registerBool(self, name):
        self.varOffsets[name] = self.getVar("genomeSize")
        self.varTypes[name] = 'bool'
        self.setVar("genomeSize",self.getVar("genomeSize")+1)
        
    def getBool(self, ind, name):
        if self.varTypes.has_key(name) and self.varTypes[name] == 'bool':
            return ind[self.varOffsets[name]]
        else:
            return None

    def messageRun(self, msg):
        self.log("running")
        
        self.setVar("running",True)
        
        self.random = random.Random(self.getVar("seed"))
        
        self.pop = {}
        self.fitness = {}
        popSize = self.getVar("popSize")
        for i in range(popSize):
            self.pop[i] = [self.random.randint(0,1) for j in range(self.getVar("genomeSize"))]
        self.setVar("gen",0)
        
        self.startGen = True
        #self.outstandingInds = range(popSize)
        #self.publish('gaEval',copy.copy(self.pop))
        
    def update(self, time, dtime):
        if self.startGen:
            self.startGen = False
            popSize = self.getVar("popSize")
            self.outstandingInds = range(popSize)
            self.publish('gaEval',copy.copy(self.pop))
    
    def messageGaFitness(self, msg):
        id, fitness = msg
        if id in self.outstandingInds:
            self.fitness[id] = fitness
            self.outstandingInds.remove(id)
            
            if not self.outstandingInds:
                
                self.setVar("gen",self.getVar("gen")+1)
                numGens = self.getVar("numGens")
                if numGens < 0 or self.getVar("gen") < numGens:
                    
                    if self.getVar("fitnessSharing"):
                        self.fitness = self.shareFitness(self.pop,self.fitness)
                        
                    popSize = self.getVar("popSize")
                    
                    newPop = {}
                    for i in range(0,popSize,2):
                        p1 = self.select(self.pop,self.fitness)
                        p2 = self.select(self.pop,self.fitness)
                        
                        c1, c2 = self.cross(p1,p2)
                        
                        c1 = self.mutate(c1)
                        c2 = self.mutate(c2)
                        
                        newPop[i] = c1
                        newPop[i+1] = c2
                        
                    self.pop = self.replace(self.pop,newPop)
                    #self.outstandingInds = range(popSize)
                    #self.publish('gaEval',copy.copy(self.pop))
                    self.startGen = True
                    
                else:
                    
                    self.log("done")
                    
                    self.setVar("running",False)
                    
                    maxId = None
                    maxFitness = None
                    for id, fitness in self.fitness.iteritems():
                        if not maxFitness or fitness > maxFitness:
                            maxFitness = fitness
                            maxId = id
                            
                    self.publish("gaBest",(maxFitness,self.pop[maxId]))
                    
    def shareFitness(self, pop, fitness):
        genomeSize = self.getVar("genomeSize")
        popSize = self.getVar("popSize")
        newFitness = {}
        for id, ind in pop.iteritems():
            sumDist = 0
            for testId, testInd in pop.iteritems():
                if testId != id:
                    sumDist += float(self.hammingDistance(ind,testInd)) / float(genomeSize)
            newFitness[id] = fitness[id] * (sumDist / float(popSize-1))
        return newFitness
        
    def hammingDistance(self, ind1, ind2):
        dist = 0
        for i1, i2 in zip(ind1, ind2):
            if i1 != i2:
                dist += 1
        return dist
                
    def flip(self, prob):
        return self.random.random() < prob
    
    def replace(self, oldPop, newPop):
        method = self.getVar("replacement")
        
        if method == 'simple':
            return newPop
        
        else:
            self.error("unknown replacement method %s" % method)
    
    def mutate(self, ind):
        mutateProb = self.getVar("mutateProb")
        method = self.getVar("mutation")
        
        if method == 'simple':
            for i in range(len(ind)):
                if self.flip(mutateProb):
                    ind[i] = int(not ind[i])
            return ind
                    
        else:
            self.error("unknown mutation method %s" % method)
                    
    def cross(self, p1, p2):
        if not self.flip(self.getVar("crossoverProb")):
            return p1, p2
        
        method = self.getVar("crossover")
        if method == 'simple':
            crossPoint = self.random.randint(0,min(len(p1),len(p2))-1)
            c1 = p1[:crossPoint]
            c1.extend(p2[crossPoint:])
            c2 = p2[:crossPoint]
            c2.extend(p1[crossPoint:])
            return c1, c2
            
        else:
            self.error("unknown crossover method %s" % method)
                    
    def select(self, pop, fitness):
        method = self.getVar("selection")
        if method == 'tournament':
            i1 = self.random.randint(0,len(pop)-1)
            i2 = self.random.randint(0,len(pop)-1)
            
            if fitness[i1] > fitness[i2]:
                return pop[i1]
            else:
                return pop[i2]
            
        else:
            self.error("unknown selection method %s" % method)
