from object import Object
import random

from rect import Rect
from sceneNode import SceneNode
from colors import colors

class Map(SceneNode):
    def __init__(self, parent, type = None, name = None):
        SceneNode.__init__(self, parent, type, name)
        
        self.defaultVar("mapw",25)
        self.defaultVar("maph",25)
        self.defaultVar("blockw",5)
        self.defaultVar("blockh",5)
        self.defaultVar("groundProb",0.75)
        
    def init(self):
        mapw = self.getVar("mapw")
        maph = self.getVar("maph")
        
        self.blockTypes = []
        for i in range(mapw):
            row = []
            for j in range(maph):
                if random.random() < self.getVar("groundProb"):
                    val = 0
                else:
                    val = 1
                row.append(val)
            self.blockTypes.append(row)
        
        blocks = Object(self, name = "blocks")
        blockw = self.getVar("blockw")
        blockh = self.getVar("blockh")
        for i in range(mapw):
            for j in range(maph):
                block = Rect(self, type = "block")
                block.setVar("x",i)
                block.setVar("y",j)
                block.setVar("ch","\'~"[self.blockTypes[i][j]])
                block.setVar("pos",(i*blockw,j*blockh,0))
                block.setVar("scale",(blockw,blockh,1))
                if self.blockTypes[i][j] == 0:
                    block.setVar("color",colors["brown"])
                else:
                    block.setVar("color",colors["blue"])
                #block.setVar("texture","test.png")

    def getType(self, x, y):
        mapw = self.getVar("mapw")
        maph = self.getVar("maph")
        blockw = self.getVar("blockw")
        blockh = self.getVar("blockh")
        totalw = mapw * blockw
        totalh = maph * blockh
        
        if x < 0 or y < 0 or x >= totalw or y >= totalh:
            return 0
        
        else:
            i = int(float(x) / float(blockw))
            j = int(float(y) / float(blockh))
            return self.blockTypes[i][j]

