from sceneNode import SceneNode
from triangle import Triangle
from circle import Circle
from rect import Rect
from object import Object
from keys import keys
from bulletEmitter import BulletEmitter
from dynLoad import dynLoad
from powerup import Powerup
import vector, random

class Player(SceneNode):
    def __init__(self, parent, type = None, name = None):
        SceneNode.__init__(self, parent, type, name)
        
        self.defaultVar("maxHp",100)
        self.defaultVar("hitRadius",5)
        self.defaultVar("dspeed",150)
        self.defaultVar("maxSpeed",200)
        self.defaultVar("friction",0.5)
        self.defaultVar("useMomentum",False)
        self.defaultVar("bulletEmitter","playerBulletEmitter")
        self.defaultVar("joystick",0)
        self.defaultVar("xaxis",0)
        self.defaultVar("yaxis",1)
        self.defaultVar("fireKey","space")
        self.defaultVar("fireButton",0)
        self.defaultVar("useJoystick",True)
        self.defaultVar("joystickPresent",False)
        self.defaultVar("score",0)
        self.defaultVar("demoTimeout",5.0)
        self.defaultVar("numPowerLevels",3)
        self.defaultVar("powerupDropProb",0.1)
        self.defaultVar("powerups","gunPowerup")
        
        self.setVar("powerLevel",1)
        self.setVar("hp",self.getVar("maxHp"))
        self.setVar("vel",(0,0))
        self.setVar("acc",(0,0))
        self.setVar("anyJoystickButtons",False)
        self.setVar("firing",False)
        
        self.gfx = self.setupGfx()
        
        bulletEmitterType = self.getVar("bulletEmitter")
        self.bulletEmitters = []
        for i in range(self.getVar("numPowerLevels")):
            emitterType = "%s%d" % (bulletEmitterType,i)
            self.log("adding bulletEmitter %s" % emitterType)
            bulletEmitter = (dynLoad(bulletEmitterType) or BulletEmitter)(self,emitterType)
            self.bulletEmitters.append(bulletEmitter)
        
        self.demoStates = ['avoid']
        self.demoPos = 0
        self.demoStartTime = None
        
        self.addInitDepend("joystick")
        
    def setupGfx(self):
        gfx = SceneNode(self,"playerGfx")
        
        hitRadius = self.getVar("hitRadius")
        
        rect = Rect(gfx)
        rect.setVar("pos",(-hitRadius,-hitRadius,0))
        rect.setVar("scale",(hitRadius*2,hitRadius*2,1))
        rect.setVar("texture","player.png")
        
        #size = hitRadius * 2.2
        #tri = Triangle(gfx)
        #tri.setVar("v1",(-size,size*0.75,0))
        #tri.setVar("v2",(size,size*0.75,0))
        #tri.setVar("v3",(0,-size,0))
        #tri.setVar("color",(0,1,0))
        
        #circle = Circle(gfx)
        #circle.setVar("color",(1,1,1))
        #circle.setVar("scale",(hitRadius,hitRadius,1))
        
        return gfx
        
    def init(self):
        SceneNode.init(self)
        
        gfxServer = self.getFirstObjectOfName("gfxServer")
        if gfxServer:
            self.screenw = gfxServer.getVar("screenw")
            self.screenh = gfxServer.getVar("screenh")
            self.setVar("pos",(self.screenw/2,self.screenh*3/4,0))
            
            self.minx = 0
            self.maxx = self.screenw
            #self.miny = self.screenh * 2/3
            self.miny = 0
            self.maxy = self.screenh

        inputServer = self.getFirstObjectOfName("inputServer")
        if inputServer:
            inputServer.subscribe(self.path)
            
        self.bullets = self.parent.getChild("bullets")
        self.enemies = self.parent.getChild("enemies")
        self.powerups = self.parent.getChild("powerups")
        
        self.joystick = self.getFirstObjectOfName("joystick")
        if self.joystick.getVar("numJoysticks") > self.getVar("joystick"):
            self.setVar("joystickPresent",True)
            
    def update(self, time, dtime):
        state = self.parent.getVar("state")
        
        self.setVar("anyJoystickButtons",False)
        if self.getVar("useJoystick") and self.getVar("joystickPresent"):
            joystick = self.getVar("joystick")
            for button in range(self.joystick.getNumButtons(joystick)):
                if self.joystick.getButton(button,joystick):
                    self.setVar("anyJoystickButtons",True)
        
        if state == "game" or state == "level":
            
            if self.getVar("useJoystick") and self.getVar("joystickPresent"):
                joystick = self.getVar("joystick")
                xaxis = self.getVar("xaxis")
                yaxis = self.getVar("yaxis")
                dspeed = self.getVar("dspeed")
                dx = self.joystick.getAxis(xaxis,joystick)
                dy = self.joystick.getAxis(yaxis,joystick)
                self.setVar("acc",(dx * dspeed, dy * dspeed))
                
                fireButton = self.getVar("fireButton")
                if self.joystick.getNumButtons(joystick) > fireButton and self.joystick.getButton(fireButton,joystick):
                    self.startFiring()
                else:
                    self.stopFiring()
            
            maxSpeed = self.getVar("maxSpeed")
            friction = self.getVar("friction")
            accx, accy = self.getVar("acc")
            
            if self.getVar("useMomentum"):
                
                velx, vely = self.getVar("vel")
                velx += accx * dtime
                vely += accy * dtime
                velx = max(min(velx,maxSpeed),-maxSpeed)
                vely = max(min(vely,maxSpeed),-maxSpeed)
                velx -= velx * friction * dtime
                vely -= vely * friction * dtime
                
                posx, posy, posz = self.getVar("pos")
                lastPosx = posx
                lastPosy = posy
                posx += velx * dtime
                posy += vely * dtime
                
                if posx < self.minx or posx > self.maxx:
                    velx *= -1
                    posx = lastPosx
                if posy < self.miny or posy > self.maxy:
                    vely *= -1
                    posy = lastPosy
                    
                self.setVar("vel",(velx,vely))
                self.setVar("pos",(posx,posy,posz))
                
            else:
                posx,posy,posz = self.getVar("pos")
                
                lastPosx = posx
                lastPosy = posy
                posx += accx * dtime
                posy += accy * dtime
                
                if posx < self.minx or posx > self.maxx:
                    posx = lastPosx
                if posy < self.miny or posy > self.maxy:
                    posy = lastPosy
                self.setVar("pos",(posx,posy,posz))
                
        else:
            if not self.demoStartTime:
                self.demoStartTime = time
            if time - self.demoStartTime > self.getVar("demoTimeout"):
                self.demoStartTime = time
                self.demoPos += 1
                if self.demoPos >= len(self.demoStates):
                    self.demoPos = 0
                self.log("demo state %s" % self.demoStates[self.demoPos])
                    
            state = self.demoStates[self.demoPos]
            
            self.startFiring()
            
            if state == 'target':
                minDist = None
                minEnemy = None
                for enemy in self.enemies.children.itervalues():
                    x,y,z = enemy.getVar("pos")
                    dist = vector.dist(self.getVar("pos"),enemy.getVar("pos"))
                    if not minDist or dist < minDist and x > 0 and x < self.screenw:
                        minDist = dist
                        minEnemy = enemy
                if minEnemy:
                    enemyx, enemyy, enemyz = minEnemy.getVar("pos")
                    x,y,z = self.getVar("pos")
                    x += (enemyx - x) / 2.0 * dtime
                    y += (self.screenh * 0.8 - y) / 2.0 * dtime
                    self.setVar("pos",(x,y,z))
                    
            elif state == 'avoid':
                minDist = None
                moveTarget = (self.screenw/2,self.screenh*0.8)
                bulletRange = 100.0
                for bullet in self.bullets.children.itervalues():
                    bullet.setVar("seekTarget",0.0)
                    nowx,nowy,z = bullet.getVar("pos")
                    velx, vely = bullet.getVar("vel")
                    x = nowx + velx * 0.5
                    y = nowy + vely * 0.5
                    myx, myy, myz = self.getVar("pos")
                    dist = vector.dist((x,y),(myx,myy))
                    if dist < bulletRange and (not minDist or dist < minDist):
                        minDist = dist
                        diff = vector.sub((myx,myy),(x,y))
                        diffl = vector.length(diff)
                        dmove = vector.scale(vector.normalize(diff),(bulletRange - diffl) * 2)
                        movex, movey = vector.add((myx,myy),dmove)
                        movex = min(max(movex,self.screenw/3),self.screenw*2/3)
                        movey = min(max(movey,self.screenh/3),self.screenh*2/3)
                        moveTarget = movex,movey
                x,y,z = self.getVar("pos")
                movex,movey = moveTarget
                x += (movex - x) / 0.25 * dtime
                y += (movey - y) / 0.25 * dtime
                self.setVar("pos",(x,y,z))
        
    def messageKeyDown(self, key):
        if not self.getVar("useJoystick") or not self.getVar("joystickPresent"):
            
            accx, accy = self.getVar("acc")
            dspeed = self.getVar("dspeed")
            
            if key == keys['left']:
                accx -= dspeed
            elif key == keys['right']:
                accx += dspeed
            elif key == keys['up']:
                accy -= dspeed
            elif key == keys['down']:
                accy += dspeed
                
            elif key == keys[self.getVar("fireKey")]:
                self.startFiring()
                
            self.setVar("acc",(accx,accy))
        
    def messageKeyUp(self, key):
        if not self.getVar("useJoystick") or not self.getVar("joystickPresent"):
            accx, accy = self.getVar("acc")
            dspeed = self.getVar("dspeed")
            
            if key == keys['left']:
                accx += dspeed
            elif key == keys['right']:
                accx -= dspeed
            elif key == keys['up']:
                accy += dspeed
            elif key == keys['down']:
                accy -= dspeed
            
            elif key == keys[self.getVar("fireKey")]:
                self.stopFiring()
                
            self.setVar("acc",(accx,accy))

    def messageDamage(self, msg):
        damage, firer = msg
        self.setVar("hp",self.getVar("hp")-damage)
        self.messageResetLevel(None)
        
    def messageDidDamage(self, msg):
        damage, firer = msg
        self.setVar("score",self.getVar("score")+damage)

    def flip(self, f):
        return random.random() < f
    
    def messageKilled(self, enemy):
        if self.flip(self.getVar("powerupDropProb")):
            powerType = random.choice(self.getVar("powerups").split())
            self.log("making %s powerup" % powerType)
            powerup = (dynLoad(powerType) or Powerup)(self.powerups,powerType)
            powerup.setVar("pos",self.getObject(enemy).getVar("pos"))
            
        self.log("killed %s" % enemy)
        self.setVar("score",self.getVar("score")+150)
        self.parent.setVar("toKill",self.parent.getVar("toKill")-1)

    def startFiring(self):
        if not self.getVar("firing"):
            self.setVar("firing",True)
            #for emitter in self.bulletEmitters[:self.getVar("powerLevel")]:
                #emitter.setVar("firing",True)
            self.bulletEmitters[self.getVar("powerLevel")-1].setVar("firing",True)

    def stopFiring(self):
        if self.getVar("firing"):
            self.setVar("firing",False)
            for emitter in self.bulletEmitters:
                emitter.setVar("firing",False)
                
    def messageLevelUp(self, msg):
        self.log("level up")
        self.setVar("powerLevel",min(self.getVar("powerLevel")+1,self.getVar("numPowerLevels")))
        self.setVar("firing",True)
        self.stopFiring()
        
    def messageResetLevel(self, msg):
        self.setVar("powerLevel",1)
        self.setVar("firing",True)
        self.stopFiring()
