from object import Object
from player import Player
from enemy import Enemy
from text import Text
from background import Background
from explosionServer import ExplosionServer
from ga import Ga
from keys import keys

import binary, vector, pickle

class Shmup(Object):
    def __init__(self, parent, type = None, name = None):
        Object.__init__(self, parent, type, name)
        
        self.defaultVar("popSize",12)
        self.defaultVar("bitsPerControlFloat",3)
        self.defaultVar("waveSize",4)
        self.defaultVar("waveOffset",2.5)
        self.defaultVar("evalTime",10.0)
        self.defaultVar("nameBase","New high score!\nEnter name> ")
        self.defaultVar("name","")
        self.defaultVar("nameChars","_ ")
        self.defaultVar("maxToKill",int(self.getVar("popSize") * 2.0))
        self.defaultVar("levelTime",5.0)
        
        self.setVar("level",1)
        self.setVar("toKill",self.getVar("maxToKill"))
        
        bpcf = self.getVar("bitsPerControlFloat")
        
        self.player = Player(self)
        self.player.setVar("score",0)
        
        self.ga = Ga(self,"shmupGa")
        self.ga.subscribe(self.path)
        self.ga.setVar("popSize",self.getVar("popSize"))
        
        self.bullets = Object(self,"folder","bullets")
        self.enemies = Object(self,"folder","enemies")
        self.powerups = Object(self,"folder","powerups")
        self.background = Background(self)
        self.explosionServer = ExplosionServer(self)
        
        self.enemyList = {}
        self.pendingEnemies = []
        self.activeEnemies = []
        self.finishedEnemies = []
        self.enemySpawnTime = None
        
        self.loadHighScores()

        for i in range(self.getVar("popSize")):
            enemy = Enemy(self.enemies)
            enemy.setVar("lifetime",self.getVar("evalTime"))
            self.enemyList[i] = enemy
            
        self.setupGa()
        
        self.evalStartTime = None
        
        self.gui = Object(self,'gui')
        
        self.text = Text(self.gui)
        self.text.setVar("pos",(5,20,0))
        
        self.addInitDepend("gfxServer")
        self.addInitDepend("ga")
        self.addInitDepend("soundServer")
        
        self.testScore = False
        self.startGame = False
        self.newScore = None
        
        self.setVar("state","init")
        
    def init(self):
        self.ga.preInit()
        
        self.inputServer = self.getFirstObjectOfName("inputServer")
        if self.inputServer:
            self.inputServer.subscribe(self.path)
        
        self.gfxServer = self.getFirstObjectOfName("gfxServer")
        if self.gfxServer:
            self.screenw = self.gfxServer.getVar("screenw")
            self.screenh = self.gfxServer.getVar("screenh")
        else:
            self.screenw = 800
            self.screenh = 600
            
        self.startText = Text(self.gui)
        self.startText.setVar("pos",(self.screenw/2,self.screenh*3/4-25,0))
        self.startText.setVar("text","Press any key to start")
        self.startText.setVar("align","center")
        
        self.nameText = Text(self.gui)
        self.nameText.setVar("pos",(self.screenw/2,self.screenh*3/4-25,0))
        self.nameText.setVar("text","this shouldn't be shown")
        self.nameText.setVar("align","center")
        
        self.highScoreText = Text(self.gui)
        self.highScoreText.setVar("pos",(self.screenw/2,self.screenh/4,0))
        self.highScoreText.setVar("text","put high score table here")
        self.highScoreText.setVar("align","center")
        
        self.levelText = Text(self.gui)
        self.levelText.setVar("pos",(self.screenw/2,self.screenh/4,0))
        self.levelText.setVar("text","LEVEL 0")
        self.levelText.setVar("align","center")
        
        self.soundServer = self.getFirstObjectOfName("soundServer")
        if self.soundServer:
            self.soundServer.receiveMessage("startLoop",("bg.wav",1.0))
        
        self.reset()
        
    def loadHighScores(self):
        try:
            fp = open("dat/highscores.dat","r")
            self.highScores = pickle.load(fp)
        except:
            self.highScores = []
            for i in range(10):
                self.highScores.append(("None",0))
        
    def saveHighScores(self):
        pickle.dump(self.highScores,open('dat/highscores.dat','w'))
        
    def makeHighScoreTable(self):
        tableText = "SHMUP\nSamsara Interactive\n\n\nHigh Scores\n"
        for i, (name, score) in enumerate(self.highScores):
            tableText += "%d: %s: %d\n" % (i+1, name, score)
        return tableText
        
    def setupGa(self):
        bpcf = self.getVar("bitsPerControlFloat")
        self.ga.registerFloatBits('startx',-1.0,0.0,bpcf)
        self.ga.registerFloatBits('starty',0.0,0.5,bpcf)
        self.ga.registerFloatBits('control1x',0.0,1.0,bpcf)
        self.ga.registerFloatBits('control1y',0.0,0.5,bpcf)
        self.ga.registerFloatBits('control2x',0.0,1.0,bpcf)
        self.ga.registerFloatBits('control2y',0.0,0.5,bpcf)
        self.ga.registerFloatBits('stopx',1.0,2.0,bpcf)
        self.ga.registerFloatBits('stopy',0.0,0.5,bpcf)
        
        self.ga.registerEnum('fireType',['straight','spread','sweep','targeted'])
        
        self.ga.registerFloatBits('fireRate',0.0,1.0,bpcf)
        self.ga.registerFloatBits('bulletSpeed',0.0,1.0,bpcf)
        self.ga.registerFloatBits('seekTarget',0.0,1.0,bpcf)
        self.ga.registerFloatBits('lifetime',0.0,1.0,bpcf)
        self.ga.registerFloatBits('maxHp',0.0,1.0,bpcf)
        
        self.log("evolving %d bits" % self.ga.getVar("genomeSize"))
        self.log(self.ga.varData)
        
    def startGa(self):
        self.ga.receiveMessage('run')
        
    def reset(self):
        self.highScoreText.active = True
        self.highScoreText.setVar("text",self.makeHighScoreTable())
        
        self.setVar("level",1)
        self.setVar("toKill",self.getVar("maxToKill"))
        
        self.setVar("maxFireRate",0.5)
        self.setVar("maxBulletSpeed",200.0)
        self.setVar("maxSeekTarget",0.5)
        self.setVar("maxLifetime",10.0)
        self.setVar("maxMaxHp",10.0)
        
        self.levelText.active = False
        
        if not self.testScore or self.player.getVar('score') <= self.highScores[-1][1]:
            self.startText.active = True
            self.nameText.active = False
            self.setVar("state","waitingToStart")
        else:
            self.nameText.active = True
            self.startText.active = False
            self.nameText.setVar("text",self.getVar("nameBase") + self.getVar("name"))
            self.setVar('state','nameEntry')
            
        self.testScore = True
        if not self.ga.getVar("running"):
            self.startGa()
        
    def messageGaEval(self, pop):
        self.log("gaEval")
        
        self.evalStartTime = None
        
        bpcf = self.getVar("bitsPerControlFloat")
        
        self.pendingEnemies = []
        self.activeEnemies = []
        self.finishedEnemies = []
        
        for id, ind in pop.iteritems():
            enemy = self.enemyList[id]
            bulletEmitter = enemy.getChild('enemyBulletEmitter')
            
            startx = self.ga.getFloat(ind,'startx')
            starty = self.ga.getFloat(ind,'starty')
            control1x = self.ga.getFloat(ind,'control1x')
            control1y = self.ga.getFloat(ind,'control1y')
            control2x = self.ga.getFloat(ind,'control2x')
            control2y = self.ga.getFloat(ind,'control2y')
            stopx = self.ga.getFloat(ind,'stopx')
            stopy = self.ga.getFloat(ind,'stopy')
            
            enemy.setVar("startPos",(startx,starty))
            enemy.setVar("control1",(control1x,control1y))
            enemy.setVar("control2",(control2x,control2y))
            enemy.setVar("stopPos",(stopx,stopy))
            
            fireType = self.ga.getEnum(ind,'fireType')
            self.log("setting %s's fireType to %s" % (enemy.path, fireType))
            bulletEmitter.setVar("fireType",fireType)
            
            fireRate = self.ga.getFloat(ind,'fireRate')
            bulletSpeed = self.ga.getFloat(ind,'bulletSpeed')
            seekTarget = self.ga.getFloat(ind,'seekTarget')
            lifetime = self.ga.getFloat(ind,'lifetime')
            maxHp = self.ga.getFloat(ind,'maxHp')
            
            fireRate, bulletSpeed, seekTarget, lifetime, maxHp = vector.normalize((fireRate,bulletSpeed,seekTarget,lifetime,maxHp))
            
            maxFireRate = self.getVar("maxFireRate")
            self.log("maxFireRate %f fireRate %f" % (maxFireRate, fireRate))
            bulletEmitter.setVar('fireRate',maxFireRate * 2.0 - fireRate * maxFireRate)
            
            maxBulletSpeed = self.getVar("maxBulletSpeed")
            bulletEmitter.setVar('bulletSpeed',bulletSpeed * maxBulletSpeed + maxBulletSpeed / 2.0)
            
            maxSeekTarget = self.getVar("maxSeekTarget")
            bulletEmitter.setVar('seekTarget',seekTarget * maxSeekTarget)
            
            maxLifetime = self.getVar("maxLifetime")
            enemy.setVar('lifetime',maxLifetime * 2.0 - lifetime * maxLifetime)
            
            maxMaxHp = self.getVar("maxMaxHp")
            enemy.setVar('maxHp',int(maxHp * maxMaxHp + maxMaxHp))
            
            self.pendingEnemies.append(enemy)

    def update(self, time, dtime):
        state = self.getVar("state")
        
        if state != 'game' and state != 'level':
            for enemy in self.enemies.children.itervalues():
                if enemy.getVar("done"):
                    enemy.reset()
                    
        if state == "level":
            level = self.getVar("level")
            time = self.levelTime
            self.levelText.setVar("text","LEVEL %d\n\n%.2f" % (level, time))
            
            self.levelTime -= dtime
            if self.levelTime <= 0:
                self.levelText.active = False
                self.setVar("state","game")
                    
        if state == "game":
            hp = self.player.getVar("hp")
            maxHp = self.player.getVar("maxHp")
            score = self.player.getVar("score")
            level = self.getVar("level")
            toKill = self.getVar("toKill")
            self.text.setVar("text","hp: %d/%d\nscore: %d\nlevel: %d\nto kill: %d" % (hp, maxHp, score, level, toKill))
            
            if toKill <= 0:
                self.setVar("state","level")
                self.levelTime = self.getVar("levelTime")
                self.levelText.active = True
                for enemy in self.enemies.children.itervalues():
                    enemy.finish()
                self.bullets.clearChildren()
                self.powerups.clearChildren()
                
                self.setVar("level",self.getVar("level") + 1)
                self.setVar("toKill",self.getVar("maxToKill"))
            
                self.setVar("maxFireRate",self.getVar("maxFireRate") / 1.05)
                self.setVar("maxBulletSpeed",self.getVar("maxBulletSpeed") * 1.05)
                self.setVar("maxSeekTarget",self.getVar("maxSeekTarget") * 1.5)
                self.setVar("maxLifetime",self.getVar("maxLifetime") / 1.2)
                self.setVar("maxMaxHp",self.getVar("maxMaxHp") * 1.5)
            
            if hp <= 0:
                self.explosionServer.receiveMessage("spawn",(self.player.getVar("pos"),25,1.0))
                self.newScore = score
                
                self.reset()
            
            if (not self.enemySpawnTime or time - self.enemySpawnTime > self.getVar("enemySpawnTime")) and len(self.activeEnemies) < self.getVar("waveSize") and self.pendingEnemies:
                enemy = self.pendingEnemies[0]
                self.pendingEnemies.remove(enemy)
                self.activeEnemies.append(enemy)
                enemy.reset()
                
            for enemy in self.activeEnemies:
                if enemy.getVar("done"):
                    self.activeEnemies.remove(enemy)
                    self.finishedEnemies.append(enemy)
            
            if not self.evalStartTime:
                self.evalStartTime = time
                
            if len(self.finishedEnemies) == self.getVar("popSize"):
                for id, enemy in self.enemyList.iteritems():
                    self.ga.receiveMessage('gaFitness',(id,enemy.getFitness()))
                    
        elif state == 'waitingToStart':
            if self.startGame or self.player.getVar("anyJoystickButtons"):
                self.startGame = False
                self.setVar("state","level")
                self.bullets.clearChildren()
                self.powerups.clearChildren()
                self.player.stopFiring()
                self.startText.active = False
                self.highScoreText.active = False
                self.nameText.active = False
                self.levelText.active = True
                self.levelTime = self.getVar("levelTime")
                self.startGa()
                self.player.setVar("hp",self.player.getVar("maxHp"))
                self.player.setVar("score",0)
                self.player.setVar("pos",(self.screenw/2,self.screenh*3/4,0))
                for enemy in self.enemies.children.itervalues():
                    enemy.finish()
                
    def messageKeyDown(self, key):
        if key == keys['escape']:
            self.newScore = self.player.getVar("score")
            if self.getVar('state') != 'waitingToStart':
                self.reset()
            else:
                self.getRoot().setVar("shutdown",True)
            return
        
        if self.getVar('state') == 'waitingToStart':
            self.startGame = True
        
        elif self.getVar("state") == 'nameEntry':
            
            if key == keys['backspace'] and self.getVar("name"):
                self.setVar("name",self.getVar("name")[:-1])
                self.nameText.setVar("text",self.getVar("nameBase") + self.getVar("name"))
                
            elif key == keys['enter']:
                newScore = self.newScore
                newName = self.getVar("name").strip()
                self.setVar("name",newName)
                
                insertPos = None
                for i in range(len(self.highScores)-1,-1,-1):
                    if newScore > self.highScores[i][1]:
                        insertPos = i
                if insertPos != None:
                    self.highScores.insert(insertPos,(newName,newScore))
                    self.highScores.remove(self.highScores[-1])
                    self.saveHighScores()
                    
                self.testScore = False
                
                self.reset()

    def messageKeyCharDown(self, key):
        if self.getVar("state") == 'nameEntry' and (key.isalnum() or key in self.getVar("nameChars")):
            self.setVar("name",self.getVar("name") + key)
            self.nameText.setVar("text",self.getVar("nameBase") + self.getVar("name"))
