from object import Object
import pygame, random

class SoundServer(Object):
    def __init__(self, parent, type = None, name = None):
        Object.__init__(self,parent,type,name)
        
        self.mixer = pygame.mixer
        
        self.defaultVar("maxSounds",20)
        self.defaultVar("soundFolder","dat/sounds/")
        
        self.sounds = {}
        self.loopingChannels = {}
        
        self.addInitDepend("gfxServer")
        
    def init(self):
        pygame.init()
        pygame.mixer.init()
        
        maxSounds = self.getVar("maxSounds")
        pygame.mixer.set_num_channels(maxSounds)
        numChannels = min(maxSounds,pygame.mixer.get_num_channels())
        self.log("%d channels" % numChannels)
        
        self.channels = [pygame.mixer.Channel(i) for i in range(numChannels)]
        
    def destroy(self):
        self.channels = []
        self.sounds = {}
        
    def load(self, file):
        if not self.sounds.has_key(file):
            filename = self.getVar("soundFolder") + file
            self.log("loading %s" % filename)
            try:
                sound = pygame.mixer.Sound(filename)
            except:
                self.log("ERROR: can't load sound %s" % filename)
                return None
            self.sounds[file] = sound
            return sound
        else:
            return self.sounds[file]
        
    def play(self, file, vol, loops):
        if len(self.loopingChannels) == len(self.channels):
            return
        
        sound = self.load(file)
        if sound:
            channel = None
            for ch in self.channels:
                if not ch in self.loopingChannels.values() and not ch.get_busy():
                    channel = ch
                    break
            while not channel:
                ch = random.choice(self.channels)
                if ch not in self.loopingChannels.values():
                    channel = ch
            channel.play(sound,loops)
            channel.set_volume(vol)
            if loops:
                self.loopingChannels[file] = channel
        
    def messagePlay(self, msg):
        file, vol = msg
        self.play(file,vol,0)

    def messageStartLoop(self, msg):
        file, vol = msg
        self.play(file,vol,-1)

    def messageStopLoop(self, file):
        if self.loopingChannels.has_key(file):
            channel = self.loopingChannels.pop(file)
            channel.stop()
